/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUITRANSACTIONPRINTER_H
#define GUITRANSACTIONPRINTER_H

#include <qtextedit.h>
#include <qlistview.h>
#include <qwidgetstack.h>
#include <qpaintdevicemetrics.h>
#include <qprinter.h>

#include "guitransactionprinterbase.h"

//class GuiMain;
class TransactionItem;

/**
* Prints the transaction list. Creates a preview pixmap when shown for the first time.
* The preview uses the screen font most similar to the printer font. However, a slight mismatch in scale
* seems to be unavoidable. 
* Before printing, the printer setup dialog is called.
* The spinboxes to the left mirror the controls of the printer setup dialog. They may be set in reverse
* order, thus allowing
* the user to set the from value to the higher page number.
*/
class GuiTransactionPrinter : public guiTransactionPrinterBase
{
  Q_OBJECT
public:

  /**
  *this is the transaction printer dialog constructor
  *
  *@param main a pointer on moneypenny´s main class
  *@param name widgets title
  */
  GuiTransactionPrinter(QWidget* parent,  const char *name);

  /**
  * this is the transaction printer dialog destructor
  */
  ~GuiTransactionPrinter();

  /**
  * Creates the html text for printing and preview.
  */
  int createPrintPages();

  /**
  * prints the text on printer
  */
  void GuiTransactionPrinter::slotPrint();

  /**
  * Updates the preview. When called for the first time, it creates the preview pixmap and then 
  * calls itself again by calling update(). Once the preview pixmap is valid, it is blitted to the
  * preview frame according to the scrollbar value.
  * @param pe pointer on the QPaintEvent
  */
  void paintEvent(QPaintEvent * se);

  /**
  * Read property of QString m_AccountName.
  */
  const QString& GuiTransactionPrinter::getAccountName()const;

  /**
  * Write property of QString m_AccountName. 
  * @param _newVal QString
  */
  void GuiTransactionPrinter::setAccountName( const QString& _newVal);

  /**
  * To be called when main data have changed
  */
  void refresh();

  /**
  * Read property of QPtrList<TransactionItem> m_TransactionsList.
  */
  const QPtrList<TransactionItem>& GuiTransactionPrinter::getTransactionList()const;

  /**
  * Write property of QPtrList<TransactionItem> m_TransactionsList. 
  * @param _newVal QPtrList<TransactionItem>
  */
  void GuiTransactionPrinter::setTransactionList( const QPtrList<TransactionItem>& _newVal);

signals:

  /**
  * emitted on cancel
  */
  void cancelled();

  /**
  * emitted on help button clicked
  */
  void helpRequest(int id);
private: //private methods

  /**
  * Sends signal cancelled (connected to gui main, slotTransactions).
  */
  void reject();

  /**
  * Activates or deactivates the spin boxes.
  */
  void slotRangeChange();

  /**
  * Calls the printer setup dialog.
  */
  void slotSetup();

  /**
  * This is a function for creating the page headers.
  * @param pageNumber Number of the page
  * @param totalPages Amount of pages
  */
  QString createPageHeader(int pageNumber, int totalPages);

  /**
  * This is a function for creating the page footers.
  * @param pageNumber Number of the page
  * @param totalPages Amount of pages
  */
  QString createPageFooter(int pageNumber, int totalPages);

  /**
  * Creates a page separator for the print preview.
  */
  QString createPageSeparator();

  /**
  * Initializes printer and printer font.
  */
  void initPrinter();

  /**
  * Calculates printable area.
  */
  QRect calculatePrintRect();

  /**
  * Finds out, whether the text fits into the printable area.
  * @param painter QPainter
  * @param text QString
  * @param pageWidth width of the page
  * @param pageHeight height of the page
  */
  bool textFits(QPainter & painter, const QString & text, int pageWidth, int pageHeight);

  /**
  * Prints a single page.
  * @param painter QPainter
  * @param text QString
  * @param font QFont
  * @param printRect QRect for the printing area
  */
  int printPage(QPainter & painter, const QString & text, QFont & font, const QRect & printRect);

  /**
  * Calls QPrinter::setup and synchronizes spin boxes and printer setup dialog.
  */
  bool setupPrinter();

  /**
  * Calls update.
  */
  void slotScrollPages();

  /**
  * Creates the background bitmap for fast print preview.
  */
  void createBackgroundPixmap();

  /**
  * Returns a html compatible RGB string.
  * @param color QColor
  */
  QString colorToHtml(const QColor & color);

  /**
  * Select line color 1
  */
  virtual void slotColor1();

  /**
  * Select line color 2
  */
  virtual void slotColor2();

  /**
  * Hide or show Options
  */
  virtual void slotOptions();

  /**
  * Select a printer font
  */
  virtual void slotFont();

  /**
  * Set new page margins
  */
  virtual void slotMarginsOk();

  /**
  * Send signal help
  */
  virtual void slotHelp();

private:

  QPtrList<TransactionItem>  _TransactionList;

  /**
  * flag for valid print pages
  */
  bool _PagesValid;

  /**
  * flag for a valid pages bitmap
  */
  bool _PixmapValid;

  /**
  * flag for valid printer setup
  */
  bool _PrinterSetupOk;

  /**Page margin settings in cm*/
  double _LeftMarginCm;
  double _TopMarginCm;
  double _RightMarginCm;
  double _BottomMarginCm;

  /**
  * page size from printer
  */
  int _PageWidth;
  int _PageHeight;
  /**
  * the height of the printer font
  */
  int _PrinterFontHeight;

  /**
  * The printer. Initialized in showEvent or when required.
  */
  QPrinter * _Printer;

  /**
  * The printer font
  */
  QFont * _PrinterFont;

  /**
  * The font metrics for the printer font. Evaluated by createPrinterText.
  */
  QFontMetrics * _PrinterFontMetrics;

  /**
  * The list of pages. Contains one string per page.
  */
  QStringList _Pages;

  /**
  * pixmap for speeding up print preview.
  */
  QPixmap * _PreviewPixmap;

  /**
  * the scale for the preview
  */
  double _Scale;

  /**
  * AccountName is
  */
  QString _AccountName;

  /**
  * Backgroundcolor for uneven line numbres
  */
  QColor _Color1;

  /**
  * Backgroundcolor for even line numbres
  */
  QColor _Color2;
};

#endif
